from __future__ import annotations

import typing as t

import aiohttp

from mealieapi.model import InteractiveModel


class Token(InteractiveModel):
    name: str
    id: int
    token: str | None = None

    def dict(self, *args, **kwargs) -> dict[str, t.Any]:  # type: ignore[override]
        data = super().dict(*args, **kwargs)
        data.pop("token")
        return data

    async def delete(self) -> None:
        await self._client.delete_api_key(self.id)


class Auth(InteractiveModel):
    access_token: str
    token_type: str | None = None

    async def refresh(self):
        resp = await self._client.request("auth/refresh")
        self.access_token = resp["access_token"]
        self.token_type = resp["token_type"]

    @property
    def header(self):
        return {aiohttp.hdrs.AUTHORIZATION: f"Bearer {self.access_token}"}



import re

from mealieapi.model import BaseModel, InteractiveModel


def camel_to_snake_case(obj):
    if isinstance(obj, dict):
        for key, value in list(obj.items()):
            new_key = camel_to_snake_case(key)
            del obj[key]
            obj[new_key] = value
            if isinstance(value, dict):
                obj[new_key] = camel_to_snake_case(value)
        return obj
    if isinstance(obj, list):
        return [camel_to_snake_case(item) for item in obj]
    if isinstance(obj, str):
        return re.sub(r"(?<!^)(?=[A-Z])", "_", obj).lower()
    raise ValueError(f"Unexpected object {obj!r}")


class File(InteractiveModel):
    file_token: str

    async def download(self) -> bytes:
        return await self._client.download_file(self.file_token)


class DebugInfo(BaseModel):
    production: bool | None = None
    version: str | None = None
    demo_status: bool | None = None
    api_port: int | None = None
    api_docs: bool | None = None
    db_type: str | None = None
    db_url: str | None = None
    default_group: str | None = None


class DebugStatistics(BaseModel):
    total_recipes: int | None = None
    total_users: int | None = None
    total_groups: int | None = None
    uncategorized_recipes: int | None = None
    untagged_recipes: int | None = None


class DebugVersion(BaseModel):
    production: bool | None = None
    version: str | None = None
    demo_status: bool | None = None



from datetime import datetime

from mealieapi.model import InteractiveModel


class Backup(InteractiveModel):
    name: str
    date: datetime | None = None

    async def download(self) -> bytes:
        return await self._client.download_backup(self.name)

    async def delete(self) -> None:
        await self._client.delete_backup(self.name)



import typing as t

from pydantic import BaseModel as BM

if t.TYPE_CHECKING:
    from mealieapi.client import MealieClient


class BaseModel(BM):
    pass


class InteractiveModel(BaseModel):
    def __init__(self, *args, _client: "MealieClient", **kwargs):
        object.__setattr__(self, "_client", _client)
        super().__init__(*args, **kwargs)



from __future__ import annotations

import typing as t
from datetime import datetime

import slugify

from mealieapi.const import YEAR_MONTH_DAY
from mealieapi.model import InteractiveModel


class Meal(InteractiveModel):
    name: str
    description: str

    @property
    def slug(self) -> str:
        return slugify.slugify(self.name)

    def dict(self, *args, **kwargs) -> dict[str, t.Any]:
        data = super().dict(*args, **kwargs)
        data.update(slug=self.slug)
        return data

    async def get_recipe(self):
        pass


class MealPlanDay(InteractiveModel):
    date: datetime
    meals: list[Meal]

    def dict(self, *args, **kwargs) -> dict[str, t.Any]:  # type: ignore[override]
        data = super().dict(*args, **kwargs)
        data["date"] = data["date"].strftime(YEAR_MONTH_DAY)
        return data


class MealPlan(InteractiveModel):
    group: str
    end_date: datetime
    start_date: datetime
    meals: list[Meal]
    id: int | None = None
    shopping_list: int | None = None

    def dict(self, *args, **kwargs) -> dict[str, t.Any]:  # type: ignore[override]
        data = super().dict(*args, **kwargs)
        data.pop("id")
        data.pop("shopping_list")
        data["end_date"] = data["end_date"].strftime(YEAR_MONTH_DAY)
        data["start_date"] = data["start_date"].strftime(YEAR_MONTH_DAY)
        return data


class Ingredient(InteractiveModel):
    title: str
    text: str
    quantity: int
    checked: bool


class ShoppingList(InteractiveModel):
    name: str
    group: str
    items: list[Ingredient]
    id: int | None = None

    def dict(self, *args, **kwargs) -> dict[str, t.Any]:  # type: ignore[override]
        data = super().dict(*args, **kwargs)
        data.pop("id")
        return data

    async def toggle_checked(self, index: int) -> "ShoppingList":
        self.items[index].checked = not self.items[index].checked
        return await self.update()

    def length(self) -> int:
        return len(self.items)

    async def create(self) -> "ShoppingList":
        return await self._client.create_shopping_list(self)

    async def update(self) -> "ShoppingList":
        if self.id is not None:
            return await self._client.update_shopping_list(self.id, self)
        raise ValueError("Missing required attribute id")

    async def delete(self) -> None:
        if self.id is not None:
            await self._client.delete_shopping_list(self.id)
        else:
            raise ValueError("Missing required attribute id")



import typing as t
from datetime import datetime
from zipfile import ZipFile

import slugify

from mealieapi.const import YEAR_MONTH_DAY, YEAR_MONTH_DAY_HOUR_MINUTE_SECOND
from mealieapi.model import BaseModel, InteractiveModel

if t.TYPE_CHECKING:
    from mealieapi.users import User


class RecipeImage(InteractiveModel):
    recipe_slug: str
    image: int


class RecipeAsset(InteractiveModel):
    recipe_slug: str
    file_name: str
    name: str | None = None
    icon: str | None = None

    async def content(self) -> bytes:
        return await self._client.get_asset(self.recipe_slug, self.file_name)


class RecipeNutrition(BaseModel):
    calories: float | None = None
    fatContent: float | None = None
    proteinContent: float | None = None
    carbohydrateContent: float | None = None
    fiberContent: float | None = None
    sodiumContent: float | None = None
    sugarContent: float | None = None


class RecipeComment(InteractiveModel):
    recipe_slug: str
    text: str
    id: int
    uuid: str
    date_added: datetime
    user: "User"

    def dict(self, *args, **kwargs) -> dict[str, t.Any]:  # type: ignore[override]
        return {"text": self.text}

    async def create(self) -> "RecipeComment":
        return await self._client.create_recipe_comment(self.recipe_slug, self)

    async def update(self, text: str) -> "RecipeComment":
        new_comment = self.copy()
        new_comment.text = text
        return await self._client.update_recipe_comment(
            self.recipe_slug, self.id, new_comment
        )

    async def delete(self) -> None:
        await self._client.delete_recipe_comment(self.recipe_slug, self.id)


class Recipe(InteractiveModel):
    name: str
    description: str | None = None
    image: str | None = None
    recipe_yield: str | None = None
    recipe_ingredient: list[str] | None = None
    recipe_instructions: list[dict[str, str]] | None = None
    tags: list[str] | None = None
    recipe_category: list[str] | None = None
    notes: list[dict[str, str]] | None = None
    rating: int | None = None
    extras: dict[str, str] | None = None
    id: int | None = None
    settings: dict[str, bool] | None = None
    total_time: str | None = None
    prep_time: str | None = None
    perform_time: str | None = None
    nutrition: RecipeNutrition | None = None
    date_added: datetime | None = None
    date_updated: datetime | None = None
    org_url: str | None = None
    tools: list | None = None
    assets: list[RecipeAsset] | None = None
    comments: list[RecipeComment] | None = None

    @property
    def slug(self) -> str:
        return slugify.slugify(self.name)

    def dict(self, *args, **kwargs) -> dict[str, t.Any]:  # type: ignore[override]
        data = super().dict(*args, **kwargs)
        data.update(slug=self.slug)
        if data["date_added"]:
            data["date_added"] = data["date_added"].strftime(YEAR_MONTH_DAY)
        if data["date_updated"]:
            data["date_updated"] = data["date_updated"].strftime(
                YEAR_MONTH_DAY_HOUR_MINUTE_SECOND
            )
        return data

    async def create(self) -> "Recipe":
        return await self._client.create_recipe(self)

    async def delete(self) -> "Recipe":
        return await self._client.delete_recipe(self.slug)

    async def get_asset(self, file_name: str):
        return await self._client.get_asset(self.slug, file_name)

    async def get_image(self, _type="original") -> bytes | None:
        """
        Gets the image for the recipe.
        Valid types are :code:`original`, :code:`min-original`, and :code:`tiny-original`
        """
        if self.image is not None:
            return await self._client.get_image(self.slug, _type)
        return None

    async def push_changes(self) -> "Recipe":
        return await self._client.update_recipe(self)

    async def get_zip(self) -> ZipFile:
        return await self._client.get_recipe_zip(self.slug)

    async def refresh(self) -> None:
        recipe = await self._client.get_recipe(self.slug)
        for attr in dir(recipe):
            if not attr.startswith("_"):
                setattr(self, attr, getattr(recipe, attr))

    def __repr__(self):
        return f"<Recipe {self.slug!r}>"


class RecipeTag(InteractiveModel):
    id: int
    name: str
    recipes: list[Recipe] | None = None

    @property
    def slug(self) -> str:
        return slugify.slugify(self.name)

    def dict(self, *args, **kwargs) -> dict[str, t.Any]:  # type: ignore[override]
        data = super().dict(*args, **kwargs)
        data.pop("id")
        return data

    async def update(self, new_name: str) -> "RecipeTag":
        return await self._client.update_tag(self.slug, new_name)

    async def delete(self):
        await self._client.delete_tag(self.id)


class RecipeCategory(InteractiveModel):
    id: int
    name: str
    recipes: list[Recipe] | None = None

    @property
    def slug(self):
        return slugify.slugify(self.name)

    def dict(self, *args, **kwargs) -> dict[str, t.Any]:  # type: ignore[override]
        data = super().dict(*args, **kwargs)
        data.pop("id")
        return data



from __future__ import annotations

import io
import typing as t
from datetime import timedelta

from mealieapi.auth import Token
from mealieapi.meals import MealPlan, ShoppingList
from mealieapi.model import InteractiveModel
from mealieapi.recipes import Recipe, RecipeCategory


class User(InteractiveModel):
    username: str
    full_name: str
    email: str
    admin: bool
    group: str
    id: int
    favorite_recipes: list[Recipe] | None = None
    tokens: list[Token] | None = None
    password: str | None = None

    def dict(self, *args, **kwargs) -> dict[str, t.Any]:  # type: ignore[override]
        data = super().dict(*args, **kwargs)
        data.pop("tokens")
        data.pop("password")
        return data

    async def create(self) -> "User":
        return await self._client.create_user(self)

    async def update(self) -> "User":
        return await self._client.update_user(self.id, self)

    async def delete(self) -> None:
        await self._client.delete_user(self.id)

    async def reset_password(self) -> None:
        await self._client.reset_password(self.id)

    async def update_password(self, new_password: str):
        if self.password:
            await self._client.update_password(self.id, self.password, new_password)
        else:
            raise ValueError("Missing password attribute, required to change password.")

    async def favorites(self) -> list[Recipe] | None:
        return await self._client.get_favorites(self.id)

    async def add_favorite(self, recipe_slug: str) -> None:
        await self._client.add_favorite(self.id, recipe_slug)

    async def remote_favorite(self, recipe_slug: str) -> None:
        await self._client.remove_favorite(self.id, recipe_slug)

    async def image(self) -> bytes:
        return await self._client.get_user_image(self.id)

    async def update_image(self, image: io.BytesIO) -> bytes:
        return await self._client.update_user_image(self.id, image)


class Group(InteractiveModel):
    name: str
    id: int | None = None
    categories: list[RecipeCategory] | None = None
    webhook_urls: list[str] | None = None
    webhook_time: timedelta | None = None
    users: list[User] | None = None
    mealplans: list[MealPlan] | None = None
    shopping_lists: list[ShoppingList] | None = None
    webhook_enable: bool | None = None

    def dict(self, *args, **kwargs) -> dict[str, t.Any]:  # type: ignore[override]
        data = super().dict(*args, **kwargs)
        data.pop("id")
        data.pop("categories")
        data.pop("users")
        data.pop("mealplans")
        data.pop("shopping_lists")
        return data

    async def create(self) -> "Group":
        return await self._client.create_group(self)

    async def update(self) -> "Group":
        if self.id is not None:
            return await self._client.update_group(self.id, self)
        raise ValueError("Missing required parameter id")

    async def delete(self) -> None:
        if self.id is not None:
            await self._client.delete_group(self.id)
        else:
            raise ValueError("Missing required parameter id")


class UserSignup(InteractiveModel):
    name: str
    admin: bool
    token: str | None = None

    def dict(self, *args, **kwargs) -> dict[str, t.Any]:  # type: ignore[override]
        data = super().dict(*args, **kwargs)
        data.pop("token")
        return data

    async def delete(self) -> None:
        await self._client.delete_signup_token(self.token)

    async def signup(self, user: User) -> User:
        if self.token:
            return await self._client.signup_with_token(self.token, user)
        raise ValueError("Tried to signup user without signup token")
